using System;
using System.Collections.Generic;
using com.agilefaqs.rft.domain;
using com.agilefaqs.rft.suggestions.strategies;
using NUnit.Framework;
using Rhino.Mocks;
using Rhino.Mocks.Constraints;

namespace com.agilefaqs.rft.suggestions
{
    [TestFixture]
    public class IdentitySuggestionOneStrategyTest
    {
        private const String COUNTRY_NAME = "india";
        private const String COUNTRY_MONIKER = "indian";
        private const String AT = "@";
        private const String FIRST_NAME = "Naresh";
        private const String LAST_NAME = "Jain";
        private static readonly String CURRENT_YEAR = DateTime.Now.Year.ToString();
        private readonly Country country = new Country("IN", COUNTRY_NAME, COUNTRY_MONIKER);

        private SuggestionParam suggestionParams;
        private ChildStrategyParam childStrategyParam;
        private MockRepository m;
        private SimpleTokenStrategy simpleTokenStrategy;
        private RandomNumberBasedStrategy randomNumberBasedStrategy;
        private YearBasedStrategy yearBasedStrategy;
        private ChildSuggestionFactory childSuggestionFactory;
        private IdentitySuggestionOneStrategy strategyOne;
        private readonly string CITY = "mumbai";
        private const String RANDOM_NUMBER = "10001";

        [SetUp]
        public void SetUp()
        {
            suggestionParams = new SuggestionParam(new LocationInformation(country, CITY), FIRST_NAME, LAST_NAME);
            childStrategyParam =
                new ChildStrategyParam(suggestionParams.ValidFirstNameToken, suggestionParams.CountryName,
                                       suggestionParams.ValidLastNameToken);
            m = new MockRepository();

            RandomNumberGenerator rng = new RandomNumberGenerator();
            UserService userService = m.Stub<UserService>();
            DomainNameService dns = m.Stub<DomainNameService>();
            SuggestionsUtil suggestionUtil = m.Stub<SuggestionsUtil>(dns);
            simpleTokenStrategy = m.StrictMock<SimpleTokenStrategy>(userService, suggestionUtil);
            randomNumberBasedStrategy = m.StrictMock<RandomNumberBasedStrategy>(userService, suggestionUtil, rng);
            yearBasedStrategy = m.StrictMock<YearBasedStrategy>(userService, suggestionUtil);

            childSuggestionFactory = m.StrictMock<ChildSuggestionFactory>();
            List<ChildSuggestionStrategy> childStrategies = new List<ChildSuggestionStrategy>();

            childStrategies.Add(simpleTokenStrategy);
            childStrategies.Add(yearBasedStrategy);
            childStrategies.Add(randomNumberBasedStrategy);
            Expect.Call(childSuggestionFactory.getChildStrategiesForSuggestionOne()).Return(childStrategies);
            m.Replay(childSuggestionFactory);
            strategyOne = new IdentitySuggestionOneStrategy(childSuggestionFactory);
            m.Verify(childSuggestionFactory);
        }

        public virtual void stopRecordingAndReplay()
        {
            m.ReplayAll();
        }

        [TearDown]
        public virtual void verifyMocks()
        {
            m.VerifyAll();
        }

        [Test]
        public virtual void shouldGenerateOfType_FirstName_AT_LastName_DOT_COM_IfAvailable()
        {
            String suggestion = String.Concat(String.Concat(String.Concat(FIRST_NAME, AT), LAST_NAME), ".com");
            Expect.Call(simpleTokenStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(suggestion);
            stopRecordingAndReplay();
            String Identity = strategyOne.getIdentity(suggestionParams);
            Assert.AreEqual(Identity, suggestion);
        }


        [Test]
        public virtual void shouldGenerateOfType_FirstNameYear_AT_LastName_DOT_COM_IfFirstSuggestionIsUnavailable()
        {
            String suggestion =
                String.Concat(String.Concat(String.Concat(String.Concat(FIRST_NAME, CURRENT_YEAR), AT), LAST_NAME),
                              ".com");
            Expect.Call(simpleTokenStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(null);
            Expect.Call(yearBasedStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(suggestion);
            stopRecordingAndReplay();
            String Identity = strategyOne.getIdentity(suggestionParams);
            Assert.AreEqual(Identity, suggestion);
        }


        [Test]
        public virtual void
            shouldGenerateOfType_FirstNameRandomNumber_AT_LastName_DOT_COM_IfSecondSuggestionIsUnavailable()
        {
            String suggestion =
                String.Concat(String.Concat(String.Concat(String.Concat(FIRST_NAME, RANDOM_NUMBER), AT), LAST_NAME),
                              ".com");
            Expect.Call(simpleTokenStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(null);
            Expect.Call(yearBasedStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(null);
            Expect.Call(randomNumberBasedStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(suggestion);
            stopRecordingAndReplay();
            String Identity = strategyOne.getIdentity(suggestionParams);
            Assert.AreEqual(Identity, suggestion);
        }
    }
}