using System;
using System.Collections.Generic;
using com.agilefaqs.rft.domain;
using com.agilefaqs.rft.suggestions.strategies;
using NUnit.Framework;
using Rhino.Mocks;
using Rhino.Mocks.Constraints;

namespace com.agilefaqs.rft.suggestions
{
    [TestFixture]
    public class IdentitySuggestionTwoStrategyTest
    {
        private const String COUNTRY_NAME = "india";
        private const String COUNTRY_MONIKER = "indian";
        private const String AT = "@";
        private const String FIRST_NAME = "Naresh";
        private const String LAST_NAME = "Jain";
        private static readonly String LAST_NAME_INTIAL = Convert.ToString(LAST_NAME[0]);
        private static readonly String CURRENT_YEAR = DateTime.Now.Year.ToString();
        private readonly Country country = new Country("IN", COUNTRY_NAME, COUNTRY_MONIKER);

        private SuggestionParam suggestionParams;
        private ChildStrategyParam childStrategyParam;
        private MockRepository m;
        private SimpleTokenStrategy simpleTokenStrategy;
        private RandomNumberBasedStrategy randomNumberBasedStrategy;
        private YearBasedStrategy yearBasedStrategy;
        private LastNameInitialStrategy lastNameInitialStrategy;
        private AbstractChildStrategy lastNameInitialYearBasedStrategy;
        private AbstractChildStrategy firstNameAppendLastNameStrategy;
        private ChildSuggestionFactory childSuggestionFactory;
        private IdentitySuggestionTwoStrategy strategyTwo;
        private const String RANDOM_NUMBER = "10001";

        [SetUp]
        public void SetUp()
        {
            suggestionParams = new SuggestionParam(new LocationInformation(country, "mumbai"), FIRST_NAME, LAST_NAME);
            childStrategyParam =
                new ChildStrategyParam(suggestionParams.ValidFirstNameToken, suggestionParams.CountryName,
                                       suggestionParams.ValidLastNameToken);
            m = new MockRepository();

            RandomNumberGenerator rng = new RandomNumberGenerator();
            UserService userService = m.Stub<UserService>();
            DomainNameService dns = m.Stub<DomainNameService>();
            SuggestionsUtil suggestionUtil = m.Stub<SuggestionsUtil>(dns);
            simpleTokenStrategy = m.StrictMock<SimpleTokenStrategy>(userService, suggestionUtil);
            randomNumberBasedStrategy = m.StrictMock<RandomNumberBasedStrategy>(userService, suggestionUtil, rng);
            yearBasedStrategy = m.StrictMock<YearBasedStrategy>(userService, suggestionUtil);
            lastNameInitialStrategy = m.StrictMock<LastNameInitialStrategy>(userService, suggestionUtil);
            lastNameInitialYearBasedStrategy =
                m.StrictMock<LastNameInitialYearBasedStrategy>(userService, suggestionUtil);
            firstNameAppendLastNameStrategy = m.StrictMock<FirstNameAppendLastNameStrategy>(userService, suggestionUtil);

            childSuggestionFactory = m.StrictMock<ChildSuggestionFactory>();
            List<ChildSuggestionStrategy> childStrategies = new List<ChildSuggestionStrategy>();

            childStrategies.Add(simpleTokenStrategy);
            childStrategies.Add(firstNameAppendLastNameStrategy);
            childStrategies.Add(lastNameInitialStrategy);
            childStrategies.Add(yearBasedStrategy);
            childStrategies.Add(lastNameInitialYearBasedStrategy);
            childStrategies.Add(randomNumberBasedStrategy);
            Expect.Call(childSuggestionFactory.getStrategies()).Return(childStrategies);

            m.Replay(childSuggestionFactory);
            strategyTwo = new IdentitySuggestionTwoStrategy(childSuggestionFactory);
            m.Verify(childSuggestionFactory);
        }

        [Test]
        public virtual void shouldGenerateOfType_FirstName_AT_Country_DOT_COM_IfAvailable()
        {
            String suggestion = String.Concat(String.Concat(String.Concat(FIRST_NAME, AT), COUNTRY_NAME), ".com");
            Expect.Call(simpleTokenStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(suggestion);
            stopRecordingAndReplay();
            String Identity = strategyTwo.getIdentity(suggestionParams);
            Assert.AreEqual(Identity, suggestion);
        }

        [Test]
        public virtual void shouldGenerateOfType_FirstNameLastname_AT_Country_DOT_COM_IfFirstSuggestionIsUnavailable()
        {
            String suggestion =
                String.Concat(String.Concat(String.Concat(String.Concat(FIRST_NAME, LAST_NAME), AT), COUNTRY_NAME),
                              ".com");
            Expect.Call(simpleTokenStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(null);
            Expect.Call(firstNameAppendLastNameStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(suggestion);
            stopRecordingAndReplay();
            String Identity = strategyTwo.getIdentity(suggestionParams);
            Assert.AreEqual(Identity, suggestion);
        }


        [Test]
        public virtual void
            shouldGenerateOfType_FirstNameLastNameInitial_AT_COUNTRY_NAME_DOT_COM_IfSecondSuggestionIsUnavailable()
        {
            String suggestion =
                String.Concat(
                    String.Concat(String.Concat(String.Concat(FIRST_NAME, LAST_NAME_INTIAL), AT), COUNTRY_NAME), ".com");
            Expect.Call(simpleTokenStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(null);
            Expect.Call(firstNameAppendLastNameStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(null);
            Expect.Call(lastNameInitialStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(suggestion);
            stopRecordingAndReplay();
            String Identity = strategyTwo.getIdentity(suggestionParams);
            Assert.AreEqual(Identity, suggestion);
        }


        [Test]
        public virtual void shouldGenerateOfType_FirstNameYear_AT_COUNTRY_NAME_DOT_COM_IfThirdSuggestionIsUnavailable()
        {
            String suggestion =
                String.Concat(String.Concat(String.Concat(String.Concat(FIRST_NAME, CURRENT_YEAR), AT), COUNTRY_NAME),
                              ".com");
            Expect.Call(simpleTokenStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(null);
            Expect.Call(firstNameAppendLastNameStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(null);
            Expect.Call(lastNameInitialStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(null);
            Expect.Call(yearBasedStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(suggestion);
            stopRecordingAndReplay();
            String Identity = strategyTwo.getIdentity(suggestionParams);
            Assert.AreEqual(Identity, suggestion);
        }


        [Test]
        public virtual void
            shouldGenerateOfType_FirstNameLastNameInitialYear_AT_COUNTRY_NAME_DOT_COM_IfFourthSuggestionIsUnavailable()
        {
            String suggestion =
                String.Concat(
                    String.Concat(
                        String.Concat(String.Concat(String.Concat(FIRST_NAME, LAST_NAME_INTIAL), CURRENT_YEAR), AT),
                        COUNTRY_NAME), ".com");
            Expect.Call(simpleTokenStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(null);
            Expect.Call(firstNameAppendLastNameStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(null);
            Expect.Call(lastNameInitialStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(null);
            Expect.Call(yearBasedStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(null);
            Expect.Call(lastNameInitialYearBasedStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(suggestion);
            stopRecordingAndReplay();
            String Identity = strategyTwo.getIdentity(suggestionParams);
            Assert.AreEqual(Identity, suggestion);
        }


        [Test]
        public virtual void shouldGenerateOfType_FirstNameRandom_AT_COUNTRY_NAME_DOT_COM_IfFifthSuggestionIsUnavailable()
        {
            String suggestion =
                String.Concat(String.Concat(String.Concat(String.Concat(FIRST_NAME, RANDOM_NUMBER), AT), COUNTRY_NAME),
                              ".com");
            Expect.Call(simpleTokenStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(null);
            Expect.Call(firstNameAppendLastNameStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(null);
            Expect.Call(lastNameInitialStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(null);
            Expect.Call(yearBasedStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(null);
            Expect.Call(lastNameInitialYearBasedStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(null);
            Expect.Call(randomNumberBasedStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(suggestion);
            stopRecordingAndReplay();
            String Identity = strategyTwo.getIdentity(suggestionParams);
            Assert.AreEqual(Identity, suggestion);
        }


        [Test]
        public virtual void shouldReturnNull_IfSixthSuggestionIsUnavailable()
        {
            Expect.Call(simpleTokenStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(null);
            Expect.Call(firstNameAppendLastNameStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(null);
            Expect.Call(lastNameInitialStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(null);
            Expect.Call(yearBasedStrategy.generateId((childStrategyParam))).Constraints(Is.Anything()).
                Return(null);
            Expect.Call(lastNameInitialYearBasedStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(null);
            Expect.Call(randomNumberBasedStrategy.generateId((childStrategyParam))).Constraints(
                Is.Anything()).Return(null);
            stopRecordingAndReplay();
            String Identity = strategyTwo.getIdentity(suggestionParams);
            Assert.IsNull(Identity);
        }

        public virtual void stopRecordingAndReplay()
        {
            m.ReplayAll();
        }

        [TearDown]
        public virtual void verifyMocks()
        {
            m.VerifyAll();
        }
    }
}